/*
  Copyright (C) 2009 AIST-CNRS

  Author: Florent Lamiraux
*/

#include "hppOpenHRP/parserOpenHRPKineoDeviceHrp2.h"
#include "hrp2_14_path.h"
#include "hppModel/hppJoint.h"
#include "hppModel/hppBody.h"

#if DEBUG==2
#undef NDEBUG
#define ODEBUG2(x) std::cout << "CparserOpenHRPKineoDeviceHrp2:" << x << std::endl
#define ODEBUG1(x) std::cerr << "CparserOpenHRPKineoDeviceHrp2:" << x << std::endl
#elif DEBUG==1
#undef NDEBUG
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "CparserOpenHRPKineoDeviceHrp2:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

//=========================================================================

CparserOpenHRPKineoDeviceHrp2::CparserOpenHRPKineoDeviceHrp2( ModelInfo_ptr InOpenHRPModelInfo) :
  CparserOpenHRPKineoDevice(InOpenHRPModelInfo)
{
}

//=========================================================================

CparserOpenHRPKineoDeviceHrp2::~CparserOpenHRPKineoDeviceHrp2()
{
}

//=========================================================================

void CparserOpenHRPKineoDeviceHrp2::parser(ChppHumanoidRobotShPtr& io_device)
{
  if (!io_device){
    string modelName = openHRPModelInfo()->getCharObject()->name() ;
    io_device = ChppHumanoidRobot::create(modelName);
  }

  if (!io_device) {
    cerr << "  CparserOpenHRPKineoDevice::parser : Failed to initialize hppDevice" << endl ; 
    io_device.reset();
    return;
  }
 
  if (buildHppDevice(io_device) != KD_OK) {
    cerr << " CparserOpenHRPKineoDevice::parser :Failed to load model"  << endl;
    io_device.reset() ;
    return;
  }

  #ifdef _HUMANOID_DYNAMIC_MULTIBODY_H_ /*defined in dynamicsJRLJapan/HumanoidDynamicMultiBody.h */
  /*
    If implementation of dynamic part of ChppHumanoidRobot is done by dynamicsJRLJapan,
    we need to read a data file in order to complete initialization of the robot.
  */
  dynamicsJRLJapan::HumanoidDynamicMultiBody *humanoidDynamicRobot;
  
  humanoidDynamicRobot = dynamic_cast<dynamicsJRLJapan::HumanoidDynamicMultiBody*>(io_device.get());

  if (!humanoidDynamicRobot) {
    ODEBUG1(":parser: robot is not of type dynamicsJRLJapan::HumanoidDynamicMultiBody");
    ODEBUG1(":parser: cannot complete initialization.");
    io_device.reset(); 
    return;
  }
  else {
    std::string directory(HRP2_14_DATADIR);
    std::string filename("/HRP2Specificities.xml");
    std::string fullPath = directory + filename;
    ODEBUG2(":parser: loading file " << fullPath);
    humanoidDynamicRobot->SetHumanoidSpecificitiesFile(fullPath);

    std::string aProperty("FileJointRank");
    std::string JointRank="/HRP2LinkJointRank.xml";
    fullPath = directory + JointRank;
    humanoidDynamicRobot->setProperty(aProperty, fullPath);

    // We need to give names to dynamicsJRLJapan::Joint objects.
    for (unsigned int iJoint=0; 
	 iJoint < io_device->countJointComponents();
	 iJoint++) {
      CkppJointComponentShPtr kppJoint =
	io_device->jointComponent(iJoint);

      CjrlJoint* jrlJoint = io_device->kppToHppJoint(kppJoint)->jrlJoint();
      dynamicsJRLJapan::Joint* jrlJapanJoint=dynamic_cast<dynamicsJRLJapan::Joint*>(jrlJoint);

      if (!jrlJapanJoint) {
	ODEBUG1(":parser: Joint is not of type dynamicsJRLJapan::Joint.");
	io_device.reset(); 
	return;
      }
      std::string jointName = kppJoint->name();
      jrlJapanJoint->setName(jointName);
      // Reference waist in CjrlHumanoidRobot class
      if (jointName == "WAIST") {
	ODEBUG2("parser: setting waist joint.");
	humanoidDynamicRobot->waist(jrlJoint);
      }
    }
  }
#endif
    // Initialize dynamic part of robot.
  io_device->initialize();

  ODEBUG2("io_device after initialization: " << *(io_device.get()));

}
